@@ -8,7 +8,7 @@
     @EXPORT = qw(); # Do NOT @EXPORT anything.
     $VERSION = sprintf("%d.%02d", q$Revision: 11.12 $ =~ /(\d+)\.(\d+)/o);
 
-#   $Id: ExampleP.pm,v 11.12 2004/01/07 17:38:51 timbo Exp $
+#   $Id: ExampleP.pm 3719 2006-03-27 09:32:36Z timbo $
 #
 #   Copyright (c) 1994,1997,1998 Tim Bunce
 #
@@ -61,6 +61,7 @@
         my ($outer, $dbh) = DBI::_new_dbh($drh, { Name => $dbname });
         $dbh->STORE('Active', 1);
         $dbh->{examplep_get_info} = {};
+        $dbh->{Name} = $dbname;
         return $outer;
     }
 
@@ -200,7 +201,7 @@
 
 
     sub ping {
-	return 2;	# used by t/80proxy.t
+	(shift->FETCH('Active')) ? 2 : 0;    # the value 2 is checked for by t/80proxy.t
     }
 
 
