@@ -6,17 +6,31 @@ DBI::Profile - Performance profiling and benchmarking for the DBI
 
 =head1 SYNOPSIS
 
-  use DBI;
+The easiest way to enable DBI profiling is to set the DBI_PROFILE
+environment variable to 2 and then run your code as usual:
 
-  $h->{Profile} = ... ;
+  DBI_PROFILE=2 prog.pl
+
+This will profile your program and then output a textual summary
+grouped by query.  You can also enable profiling by setting the
+Profile attribute of any DBI handle:
+
+  $dbh->{Profile} = 2;
+
+Other values are possible - see L<"ENABLING A PROFILE"> below.
 
 =head1 DESCRIPTION
 
-DBI::Profile is new and experimental and subject to change.
+DBI::Profile is fairly new and subject to change.
 
 The DBI::Profile module provides a simple interface to collect and
 report performance and benchmarking data from the DBI.
 
+For a more elaborate interface, suitable for larger programs, see
+L<DBI::ProfileDumper|DBI::ProfileDumper> and L<dbiprof|dbiprof>.
+For Apache/mod_perl applications see
+L<DBI::ProfileDumper::Apache|DBI::ProfileDumper::Apache>.
+
 =head1 OVERVIEW
 
 Performance data collection for the DBI is built around several
@@ -161,6 +175,9 @@ values are interpreted is to show the code:
     push @Path, DBIprofile_Statement        if $path & 0x02;
     push @Path, DBIprofile_MethodName       if $path & 0x04;
     push @Path, DBIprofile_MethodClass      if $path & 0x08;
+    push @Path, DBIprofile_Caller           if $path & 0x10;
+
+(The order here is subject to change and shouldn't be relied upon.)
 
 So using the value "C<1>" causes all profile data to be merged into
 a single leaf of the tree. That's useful when you just want a total.
@@ -205,56 +222,86 @@ A reference to a hash containing the collected profile data.
 
 =head2 Path
 
-The Path value is used to control where the profile for a method
-call will be merged into the collected profile data.  Whenever
-profile data is to be stored the current value for Path is used.
+The Path value is a reference to an array. Each element controls the
+value to use at the corresponding level of the profile Data tree.
 
-The value can be one of:
+The elements of Path array can be one of the following types:
 
 =over 4
 
-=item Array Reference
+=item Special Constant
 
-Each element of the array defines an element of the path to use to
-store the profile data into the C<Data> hash.
+B<DBIprofile_Statement>
 
-=item Undefined value (the default)
+Use the current Statement text. Typically that's the value of the Statement
+attribute for the handle the method was called with. Some methods, like
+commit() and rollback(), are unrelated to a particular statement. For those
+methods DBIprofile_Statement records an empty string.
 
-Treated the same as C<[ $DBI::Profile::DBIprofile_Statement ]>.
+B<DBIprofile_MethodName>
 
-=item Subroutine Reference B<NOT YET IMPLEMENTED>
+Use the name of the DBI method that the profile sample relates to.
 
-The subroutine is passed the DBI method name and the handle it was
-called on.  It should return a list of values to uses as the path.
-If it returns an empty list then the method call is not profiled.
+B<DBIprofile_MethodClass>
+
+Use the fully qualified name of the DBI method, including
+the package, that the profile sample relates to. This shows you
+where the method was implemented. For example:
+
+  'DBD::_::db::selectrow_arrayref' =>
+      0.022902s
+  'DBD::mysql::db::selectrow_arrayref' =>
+      2.244521s / 99 = 0.022445s avg (first 0.022813s, min 0.022051s, max 0.028932s)
+
+The "DBD::_::db::selectrow_arrayref" shows that the driver has
+inherited the selectrow_arrayref method provided by the DBI.
+
+But you'll note that there is only one call to
+DBD::_::db::selectrow_arrayref but another 99 to
+DBD::mysql::db::selectrow_arrayref. Currently the first
+call Pern't record the true location. That may change.
+
+B<DBIprofile_Caller>
+
+Use a string showing the filename and line number of the code calling the
+method, and the filename and line number of the code that called that.
+The content and format of the string may change.
+
+=item Code Reference
+
+Not yet implemented.
+
+The subroutine is passed the DBI method name and the handle it was called on.
+It should return a list of values to used at this point in the Path.  If it
+returns an empty list then the method call is not profiled.
+
+=item Attribute Specifier
+
+A string enclosed in braces, such as 'C<{Username}>', specifies that the current
+value of the corresponding database handle attribute should be used at that
+point in the Path.
+
+=item Other Values
+
+Any other values are stringified and used literally.
+
+(References, and values that begin with punctuation characters are reserved.)
 
 =back
 
-The following 'magic cookie' values can be included in the Path and will be
+Only the first 100 elements in Path are used.
 
-=over 4
+If the value of Path is anything other than an array reference,
+it is treated as if it was:
 
-=item DBIprofile_Statement
-
-Replaced with the current value of the Statement attribute for the
-handle the method was called with. If that value is undefined then
-an empty string is used.
-
-=item DBIprofile_MethodName
-
-Replaced with the name of the DBI method that the profile sample
-relates to.
-
-=back
-
-Other magic cookie values may be added in the future.
+	[ DBI::Profile::DBIprofile_Statement ]
 
 
 =head1 REPORTING
 
 =head2 Report Format
 
-The current profile data can be formatted and output using
+The current accumulated profile data can be formatted and output using
 
     print $h->{Profile}->format;
 
@@ -266,7 +313,7 @@ you can do:
 
 The default results format looks like this:
 
-  DBI::Profile: 0.001015 seconds (5 method calls) programname
+  DBI::Profile: 0.001015s 42.7% (5 calls) programname @ YYYY-MM-DD HH:MM:SS
   '' =>
       0.000024s / 2 = 0.000012s avg (first 0.000015s, min 0.000009s, max 0.000015s)
   'SELECT mode,size,name FROM table' =>
@@ -276,7 +323,13 @@ Which shows the total time spent inside the DBI, with a count of
 the total number of method calls and the name of the script being
 run, then a formated version of the profile data tree.
 
-In this example the paths in the tree are only one level deep and
+If the results are being formated when the perl process is exiting
+(which is usually the case when the DBI_PROFILE environment variable
+is used) then the percentage of time the process spent inside the
+DBI is also shown. If the process is not exiting then the percentage is
+calculated using the time between the first and last call to the DBI.
+
+In the example above the paths in the tree are only one level deep and
 use the Statement text as the value (that's the default behaviour).
 
 The merged profile data at the 'leaves' of the tree are presented
@@ -311,7 +364,7 @@ time per leaf node.
 The default method of reporting is for the DESTROY method of the
 Profile object to format the results and write them using:
 
-    DBI->trace_msg($results, 0)
+    DBI->trace_msg($results, 0);  # see $ON_DESTROY_DUMP below
 
 to write them to the DBI trace() filehandle (which defaults to
 STDERR). To direct the DBI trace filehandle to write to a file
@@ -323,6 +376,14 @@ trace level of 0. For example:
 The same effect can be achieved without changing the code by
 setting the C<DBI_TRACE> environment variable to C<0=filename>.
 
+The $DBI::Profile::ON_DESTROY_DUMP variable holds a code ref
+that's called to perform the output of the formatted results.
+The default value is:
+
+  $ON_DESTROY_DUMP = sub { DBI->trace_msg($results, 0) };
+
+Apart from making it easy to send the dump elsewhere, it can also
+be useful as a simple way to disable dumping results.
 
 =head1 CHILD HANDLES
 
@@ -332,6 +393,52 @@ then by default the statement handles created from it all contribute
 to the same merged profile data tree.
 
 
+=head1 CUSTOM DATA MANIPULATION
+
+Recall that C<$h->{Profile}->{Data}> is a reference to the collected data.
+Either to a 'leaf' array (when the Path is empty, i.e., DBI_PROFILE env var is 1),
+or a reference to hash containing values that are either further hash
+references or leaf array references.
+
+Sometimes it's useful to be able to summarise some or all of the collected data.
+The dbi_profile_merge() function can be used to merge leaf node values.
+
+=head2 dbi_profile_merge
+
+  use DBI qw(dbi_profile_merge);
+
+  $time_in_dbi = dbi_profile_merge(my $totals=[], @$leaves);
+
+Merges profile data node. Given a reference to a destination array, and zero or
+more references to profile data, merges the profile data into the destination array.
+For example:
+
+  $time_in_dbi = dbi_profile_merge(
+      my $totals=[],
+      [ 10, 0.51, 0.11, 0.01, 0.22, 1023110000, 1023110010 ],
+      [ 15, 0.42, 0.12, 0.02, 0.23, 1023110005, 1023110009 ],
+  );        
+
+$totals will then contain
+
+  [ 25, 0.93, 0.11, 0.01, 0.23, 1023110000, 1023110010 ]
+
+and $time_in_dbi will be 0.93;
+
+For example, to get the time spent 'inside' the DBI during an http request,
+your logging code run at the end of the request (i.e. mod_perl LogHandler)
+could use:
+
+  my $time_in_dbi = 0;
+  if (my $Profile = $dbh->{Profile}) { # if DBI profiling is enabled
+      $time_in_dbi = dbi_profile_merge(my $total=[], $Profile->{Data});
+      $Profile->{Data} = {}; # reset the profile data
+  }
+
+If profiling has been enabled then $time_in_dbi will hold the time spent inside
+the DBI for that handle (and any other handles that share the same profile data)
+since the last request.
+
 =head1 CUSTOM DATA COLLECTION
 
 =head2 Using The Path Attribute
@@ -349,14 +456,13 @@ The dbi_profile() function can be used to add extra sample data
 into the profile data tree. For example:
 
     use DBI;
-    use DBI::Profile (dbi_profile);
-    use Time::HiRes qw(gettimeofday);
+    use DBI::Profile (dbi_profile dbi_time);
 
-    my $t1 = gettimeofday;
+    my $t1 = dbi_time(); # floating point high-resolution time
 
     ... execute code you want to profile here ...
 
-    my $t2 = gettimeofday;
+    my $t2 = dbi_time();
     dbi_profile($h, $statement, $method, $t1, $t2);
 
 The $h parameter is the handle the extra profile sample should be
@@ -366,7 +472,8 @@ then $h->{Statement} will be used. Similarly $method is the string
 to use if the Path specifies DBIprofile_MethodName. There is no
 default value for $method.
 
-The $h->{Path} attribute is processed by dbi_profile() in the usual way.
+The $h->{Profile}{Path} attribute is processed by dbi_profile() in
+the usual way.
 
 It is recommended that you keep these extra data samples separate
 from the DBI profile data samples by using values for $statement
@@ -390,46 +497,61 @@ in the Profile Data structure for each statement.
 If a method throws an exception itself (not via RaiseError) then
 it won't be counted in the profile.
 
-If a HandleError subroutine throws an exception, rather than returning
-0 and letting RaiseError do it, then the method call won't be counted
+If a HandleError subroutine throws an exception (rather than returning
+0 and letting RaiseError do it) then the method call won't be counted
 in the profile.
 
-Time spent in DESTROY is currently not counted.
+Time spent in DESTROY is added to the profile of the parent handle.
 
 Time spent in DBI->*() methods is not counted. The time spent in
 the driver connect method, $drh->connect(), when it's called by
 DBI->connect is counted if the DBI_PROFILE environment variable is set.
 
+Time spent fetching tied variables, $DBI::errstr, is counted.
+
+Time spent in FETCH for $h->{Profile} is not counted, so getting the profile
+data doesn't alter it.
+
 DBI::PurePerl does not support profiling (though it could in theory).
 
 A few platforms don't support the gettimeofday() high resolution
-time function used by the DBI. In which case you'll get integer
-resolution time which are mostly useless.
+time function used by the DBI (and available via the dbi_time() function).
+In which case you'll get integer resolution time which is mostly useless.
+
+On Windows platforms the dbi_time() function is limited to millisecond
+resolution. Which isn't sufficiently fine for our needs, but still
+much better than integer resolution. This limited resolution means
+that fast method calls will often register as taking 0 time. And
+timings in general will have much more 'jitter' depending on where
+within the 'current millisecond' the start and and timing was taken.
 
 This documentation could be more clear. Probably needs to be reordered
 to start with several examples and build from there.  Trying to
-explain the concepts first seems to lead to many forward references.
-(Patches welcome.)
+explain the concepts first seems painful and to lead to just as
+many forward references.  (Patches welcome!)
 
 =cut
 
 
 use strict;
-use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
+use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION $ON_DESTROY_DUMP);
 use Exporter ();
 use UNIVERSAL ();
 use Carp;
 
-use DBI qw(dbi_profile dbi_profile_merge);
+use DBI qw(dbi_time dbi_profile dbi_profile_merge);
 
-$VERSION = sprintf "%d.%02d", '$Revision: 1.1 $ ' =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", '$Revision: 1.7 $ ' =~ /(\d+)\.(\d+)/;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(
     DBIprofile_Statement
     DBIprofile_MethodName
+    DBIprofile_MethodClass
+    DBIprofile_Caller
     dbi_profile
     dbi_profile_merge
+    dbi_time
 );
 @EXPORT_OK = qw(
     format_profile_thingy
@@ -438,7 +560,9 @@ $VERSION = sprintf "%d.%02d", '$Revision: 1.1 $ ' =~ /(\d+)\.(\d+)/;
 use constant DBIprofile_Statement	=> -2100000001;
 use constant DBIprofile_MethodName	=> -2100000002;
 use constant DBIprofile_MethodClass	=> -2100000003;
+use constant DBIprofile_Caller  	=> -2100000004;
 
+$ON_DESTROY_DUMP = sub { DBI->trace_msg(shift, 0) };
 
 sub new {
     my $class = shift;
@@ -455,8 +579,20 @@ sub _auto_new {
     # assigned to the Profile attribute. For example
     #	dbi:mysql(RaiseError=>1,Profile=>4/DBIx::MyProfile):dbname
     # This sub works out what to do and returns a suitable hash ref.
+    
+    my ($path, $module, @args);
 
-    my ($path, $module, @args) = split /\s*\/\s*/, $arg, -1;
+    # parse args
+    if ($arg =~ m!/!) {
+        # it's a path/module/arg/arg/arg list
+        ($path, $module, @args) = split /\s*\/\s*/, $arg, -1;
+    } elsif ($arg =~ /^\d+$/) {
+        # it's a numeric path selector
+        $path = $arg;
+    } else {
+        # it's a module name
+        $module = $arg;
+    }
 
     my @Path;
     if ($path) {
@@ -465,11 +601,15 @@ sub _auto_new {
 	push @Path, DBIprofile_Statement	if $path & 0x02;
 	push @Path, DBIprofile_MethodName	if $path & 0x04;
 	push @Path, DBIprofile_MethodClass	if $path & 0x08;
+	push @Path, DBIprofile_Caller	  	if $path & 0x10;
 	@Path = reverse @Path if $reverse;
+    } else {
+        # default Path
+        push @Path, DBIprofile_Statement;
     }
 
     if ($module) {
-	if (eval { require $module }) {
+	if (eval "require $module") {
 	  $class = $module;
 	}
 	else {
@@ -494,10 +634,17 @@ sub format {
 
     if (@$leaves) {
 	dbi_profile_merge(my $totals=[], @$leaves);
-	my ($count, $total_time) = @$totals;
+	my ($count, $time_in_dbi, undef, undef, undef, $t1, $t2) = @$totals;
 	(my $progname = $0) =~ s:.*/::;
-	$prologue .= sprintf "%f seconds (%d method calls) $progname\n",
-	    $total_time, $count if $count;
+	if ($count) {
+	    $prologue .= sprintf "%fs ", $time_in_dbi;
+	    my $perl_time = ($DBI::PERL_ENDING) ? time_in_dbi() - $^T : $t2-$t1;
+	    $prologue .= sprintf "%.2f%% ", $time_in_dbi/$perl_time*100 if $perl_time;
+	    my @lt = localtime(time);
+	    my $ts = sprintf "%d-%02d-%02d %02d:%02d:%02d",
+		1900+$lt[5], $lt[4]+1, @lt[3,2,1,0];
+	    $prologue .= sprintf "(%d calls) $progname \@ $ts\n", $count;
+	}
 
 	if (@$leaves == 1 && $self->{Data}->{DBI}) {
 	    $detail = "";	# hide it
@@ -556,15 +703,21 @@ sub format_profile_thingy {
 }
 
 
+sub on_destroy {
+    my $self = shift;
+    return unless $ON_DESTROY_DUMP;
+    return unless $self->{Data};
+    my $detail = $self->format();
+    $ON_DESTROY_DUMP->($detail) if $detail;
+}
+
 sub DESTROY {
     my $self = shift;
-    eval {
-	my $detail = $self->format() if $self->{Data};
-	DBI->trace_msg($detail, 0);
-    };
+    local $@;
+    eval { $self->on_destroy };
     if ($@) {
-	my $class = ref($self) || $self;
-	DBI->trace_msg("$class format failed: $@", 0);
+        my $class = ref($self) || $self;
+        DBI->trace_msg("$class on_destroy failed: $@", 0);
     }
 }
 
